@@ -1,18 +1,18 @@
 # Copyright Contributors to the Packit project.
 # SPDX-License-Identifier: MIT
 
-import contextlib
 import datetime
 import re
 import subprocess
 import types
 from dataclasses import dataclass
 from pathlib import Path
-from typing import Iterator, List, Optional, Tuple, Type, Union
+from typing import Generator, List, Optional, Tuple, Type, Union
 
 import rpm
 
 from specfile.changelog import Changelog, ChangelogEntry
+from specfile.context_management import ContextManager
 from specfile.exceptions import SourceNumberException, SpecfileException
 from specfile.macro_definitions import MacroDefinition, MacroDefinitions
 from specfile.macros import Macro, Macros
@@ -148,8 +148,8 @@ class Specfile:
         self._parser.parse(str(self))
         return Macros.dump()
 
-    @contextlib.contextmanager
-    def lines(self) -> Iterator[List[str]]:
+    @ContextManager
+    def lines(self) -> Generator[List[str], None, None]:
         """
         Context manager for accessing spec file lines.
 
@@ -163,8 +163,8 @@ class Specfile:
             if self.autosave:
                 self.save()
 
-    @contextlib.contextmanager
-    def macro_definitions(self) -> Iterator[MacroDefinitions]:
+    @ContextManager
+    def macro_definitions(self) -> Generator[MacroDefinitions, None, None]:
         """
         Context manager for accessing macro definitions.
 
@@ -178,8 +178,8 @@ class Specfile:
             finally:
                 lines[:] = macro_definitions.get_raw_data()
 
-    @contextlib.contextmanager
-    def sections(self) -> Iterator[Sections]:
+    @ContextManager
+    def sections(self) -> Generator[Sections, None, None]:
         """
         Context manager for accessing spec file sections.
 
@@ -200,8 +200,10 @@ class Specfile:
             return None
         return Sections.parse(self._parser.spec.parsed.splitlines())
 
-    @contextlib.contextmanager
-    def tags(self, section: Union[str, Section] = "package") -> Iterator[Tags]:
+    @ContextManager
+    def tags(
+        self, section: Union[str, Section] = "package"
+    ) -> Generator[Tags, None, None]:
         """
         Context manager for accessing tags in a specified section.
 
@@ -212,26 +214,21 @@ class Specfile:
         Yields:
             Tags in the section as `Tags` object.
         """
-        if isinstance(section, Section):
-            raw_section = section
-            parsed_section = getattr(self.parsed_sections, section.name, None)
+        with self.sections() as sections:
+            if isinstance(section, Section):
+                raw_section = section
+                parsed_section = getattr(self.parsed_sections, section.name, None)
+            else:
+                raw_section = getattr(sections, section)
+                parsed_section = getattr(self.parsed_sections, section, None)
             tags = Tags.parse(raw_section, parsed_section)
             try:
                 yield tags
             finally:
                 raw_section.data = tags.get_raw_section_data()
-        else:
-            with self.sections() as sections:
-                raw_section = getattr(sections, section)
-                parsed_section = getattr(self.parsed_sections, section, None)
-                tags = Tags.parse(raw_section, parsed_section)
-                try:
-                    yield tags
-                finally:
-                    raw_section.data = tags.get_raw_section_data()
 
-    @contextlib.contextmanager
-    def changelog(self) -> Iterator[Optional[Changelog]]:
+    @ContextManager
+    def changelog(self) -> Generator[Optional[Changelog], None, None]:
         """
         Context manager for accessing changelog.
 
@@ -250,8 +247,8 @@ class Specfile:
                 finally:
                     section.data = changelog.get_raw_section_data()
 
-    @contextlib.contextmanager
-    def prep(self) -> Iterator[Optional[Prep]]:
+    @ContextManager
+    def prep(self) -> Generator[Optional[Prep], None, None]:
         """
         Context manager for accessing %prep section.
 
@@ -270,13 +267,13 @@ class Specfile:
                 finally:
                     section.data = prep.get_raw_section_data()
 
-    @contextlib.contextmanager
+    @ContextManager
     def sources(
         self,
         allow_duplicates: bool = False,
         default_to_implicit_numbering: bool = False,
         default_source_number_digits: int = 1,
-    ) -> Iterator[Sources]:
+    ) -> Generator[Sources, None, None]:
         """
         Context manager for accessing sources.
 
@@ -288,7 +285,7 @@ class Specfile:
         Yields:
             Spec file sources as `Sources` object.
         """
-        with self.sections() as sections, self.tags(sections.package) as tags:
+        with self.sections() as sections, self.tags() as tags:
             sourcelists = [
                 (s, Sourcelist.parse(s, context=self))
                 for s in sections
@@ -307,13 +304,13 @@ class Specfile:
                 for section, sourcelist in sourcelists:
                     section.data = sourcelist.get_raw_section_data()
 
-    @contextlib.contextmanager
+    @ContextManager
     def patches(
         self,
         allow_duplicates: bool = False,
         default_to_implicit_numbering: bool = False,
         default_source_number_digits: int = 1,
-    ) -> Iterator[Patches]:
+    ) -> Generator[Patches, None, None]:
         """
         Context manager for accessing patches.
 
@@ -325,7 +322,7 @@ class Specfile:
         Yields:
             Spec file patches as `Patches` object.
         """
-        with self.sections() as sections, self.tags(sections.package) as tags:
+        with self.sections() as sections, self.tags() as tags:
             patchlists = [
                 (s, Sourcelist.parse(s, context=self))
                 for s in sections
@@ -530,7 +527,7 @@ class Specfile:
     @property
     def expanded_release(self) -> str:
         """Release string without the dist suffix with macros expanded."""
-        return self.expand(self.release)
+        return self.expand(self.release, extra_macros=[("dist", "")])
 
     def set_version_and_release(self, version: str, release: str = "1") -> None:
         """
@@ -581,7 +578,11 @@ class Specfile:
                 patches[index].comments.extend(comment.splitlines())
 
     def update_value(
-        self, value: str, requested_value: str, protected_entities: Optional[str] = None
+        self,
+        value: str,
+        requested_value: str,
+        position: int,
+        protected_entities: Optional[str] = None,
     ) -> str:
         """
         Updates a value from within the context of the spec file with a new value,
@@ -591,6 +592,7 @@ class Specfile:
         Args:
             value: Value to update.
             requested_value: Requested new value.
+            position: Position (line number) of the value in the spec file.
             protected_entities: Regular expression specifying protected tags and macro definitions,
               ensuring their values won't be updated.
 
@@ -600,8 +602,10 @@ class Specfile:
 
         @dataclass
         class Entity:
+            name: str
             value: str
             type: Type
+            position: int
             locked: bool = False
             updated: bool = False
 
@@ -611,32 +615,38 @@ class Specfile:
             re.IGNORECASE,
         )
         # collect modifiable entities
-        entities = {}
+        entities = []
         with self.macro_definitions() as macro_definitions:
-            entities.update(
-                {
-                    md.name: Entity(md.body, type(md))
+            entities.extend(
+                [
+                    Entity(
+                        md.name, md.body, type(md), md.get_position(macro_definitions)
+                    )
                     for md in macro_definitions
                     if not protected_regex.match(md.name)
                     and not md.name.endswith(")")  # skip macro definitions with options
-                }
+                ]
             )
-        # order matters here - if there is a macro definition redefining a tag,
-        # we want to update the tag, not the macro definition
         with self.tags() as tags:
-            entities.update(
-                {
-                    t.name.lower(): Entity(t.value, type(t))
+            entities.extend(
+                [
+                    Entity(t.name.lower(), t.value, type(t), t.get_position(tags))
                     for t in tags
                     if not protected_regex.match(t.name)
-                }
+                ]
             )
-        # tags can be referenced as %{tag} or %{TAG}
-        entities.update({k.upper(): v for k, v in entities.items() if v.type == Tag})
+        entities.sort(key=lambda e: e.position)
 
-        def update(value, requested_value):
+        def update(value, requested_value, position):
+            modifiable_entities = {e.name for e in entities if e.position < position}
+            # tags can be referenced as %{tag} or %{TAG}
+            modifiable_entities.update(
+                e.name.upper()
+                for e in entities
+                if e.position < position and e.type == Tag
+            )
             regex, template = ValueParser.construct_regex(
-                value, entities.keys(), context=self
+                value, modifiable_entities, context=self
             )
             m = regex.match(requested_value)
             if m:
@@ -644,33 +654,41 @@ class Specfile:
                 for grp, val in d.items():
                     if grp.startswith(SUBSTITUTION_GROUP_PREFIX):
                         continue
-                    if entities[grp].locked:
+                    # find the closest matching entity
+                    entity = [
+                        e
+                        for e in entities
+                        if e.position < position
+                        and (
+                            e.name == grp
+                            and e.type == MacroDefinition
+                            or e.name == grp.lower()
+                            and e.type == Tag
+                        )
+                    ][-1]
+                    if entity.locked:
                         # avoid infinite recursion
                         return requested_value
-                    entities[grp].locked = True
+                    entity.locked = True
                     try:
-                        entities[grp].value = update(entities[grp].value, val)
+                        entity.value = update(entity.value, val, entity.position)
                     finally:
-                        entities[grp].locked = False
-                        entities[grp].updated = True
+                        entity.locked = False
+                        entity.updated = True
                 return template.substitute(d)
             # no match, simply return the requested value
             return requested_value
 
-        result = update(value, requested_value)
+        result = update(value, requested_value, position)
         # synchronize back any changes
         with self.macro_definitions() as macro_definitions:
-            for n, v in [
-                (n, v)
-                for n, v in entities.items()
-                if v.updated and v.type == MacroDefinition
+            for entity in [
+                e for e in entities if e.updated and e.type == MacroDefinition
             ]:
-                getattr(macro_definitions, n).body = v.value
+                getattr(macro_definitions, entity.name).body = entity.value
         with self.tags() as tags:
-            for n, v in [
-                (n, v) for n, v in entities.items() if v.updated and v.type == Tag
-            ]:
-                getattr(tags, n).value = v.value
+            for entity in [e for e in entities if e.updated and e.type == Tag]:
+                getattr(tags, entity.name).value = entity.value
         return result
 
     def update_tag(
@@ -688,11 +706,13 @@ class Specfile:
               ensuring their values won't be updated.
         """
         with self.tags() as tags:
-            original_value = getattr(tags, name).value
+            tag = getattr(tags, name)
+            original_value = tag.value
+            position = tag.get_position(tags)
         # we can't use update_value() within the context manager, because any changes
         # made by it to tags or macro definitions would be thrown away
         updated_value = self.update_value(
-            original_value, value, protected_entities=protected_entities
+            original_value, value, position, protected_entities=protected_entities
         )
         with self.tags() as tags:
             getattr(tags, name).value = updated_value
